#include <cstddef>
#include <iostream>
#include <limits>

namespace SK {
	template<class T> class LogAllocator;

	template<> class LogAllocator<void> {
	public:
		typedef void value_type;
		typedef void* pointer;
		typedef const void* const_pointer;
		template<class U> struct rebind { typedef LogAllocator<U> other; };
	};

	template<class T>
	class LogAllocator {
	public:
		typedef T value_type;
		typedef T* pointer;
		typedef const T* const_pointer;
		typedef T& reference;
		typedef const T& const_reference;
		typedef std::size_t size_type;
		typedef std::ptrdiff_t difference_type;

		template<class U> struct rebind { typedef LogAllocator<U> other; };

		LogAllocator() throw() { }
		~LogAllocator() throw() { }
		template<class U> LogAllocator(const LogAllocator<U>&) throw() { }
		LogAllocator(const LogAllocator&) throw() { }

		pointer address(reference x) const { return &x; }
		const_pointer address(const_reference x) const { return &x; }

		size_type max_size() const throw()
			{ return std::numeric_limits<size_type>::max() / sizeof(value_type); }

		pointer allocate(size_type n,
				typename LogAllocator<void>::const_pointer hint = 0) {
			pointer p = static_cast<pointer>(::operator new(n * sizeof(value_type)));
			std::clog << "allocate: " << n << " Objekte an Adresse " << p << endl;
			return p;
		}

		void deallocate(pointer p, size_type n) {
			std::clog << "deallocate: " << n << " Objekte an Adresse " << p << endl;
			::operator delete(p);
		}

		void construct(pointer p, const_reference val) {
			std::clog << "construct: Objekt an Adresse " << p
				<< " mit dem Wert " << val << endl;
			new(p) T(val);
		}

		void destroy(pointer p) {
			std::clog << "destroy: Objekt an Adresse " << p << endl;
			p->~T();
		}
	};

	template<class T, class U> inline
	bool operator==(const LogAllocator<T>&, const LogAllocator<U>&) throw()
		{ return true; }

	template<class T, class U> inline
	bool operator!=(const LogAllocator<T>&, const LogAllocator<U>&) throw()
		{ return false; }
}
