#include <algorithm>
#include <functional>
#include <iostream>
#include <iterator>
#include <set>
#include "wertpapier.h"
#include "wp_objekte.h"

#include <memory>
#include "aktie.h"

using namespace std;

template<class T>
struct ptr_less : binary_function<const T*, const T*, bool> {
	bool operator()(const T* x, const T* y) const { return *x < *y; }
};

template<class T>
struct const_ptr_deref : unary_function<const T*, T> {
	const T& operator()(const T* w) const { return *w; }
};

class WKN : public unary_function<const Wertpapier*, bool> {
public:
	WKN(const string& s) : wkn(s) { }
	bool operator()(const Wertpapier* w) const { return w->gibKennNr() == wkn; }
private:
	const string wkn;
};

struct WPBez : binary_function<const Wertpapier*, string, bool> {
	bool operator()(const Wertpapier* w, const string& bez) const
		{ return w->gibBezeichnung() == bez; }
};

int main() {
	typedef set<Wertpapier*, ptr_less<Wertpapier> > WPSet;
	WPSet zeigermenge(wp, wp + n);
	transform(zeigermenge.begin(), zeigermenge.end(),
		ostream_iterator<Wertpapier>(cout, "\n"), const_ptr_deref<Wertpapier>());

	WPSet::iterator iter
		= find_if(zeigermenge.begin(), zeigermenge.end(), WKN("716460"));
	if (iter != zeigermenge.end())
		cout << "\nGefunden: " << **iter << endl;

	iter = find_if(zeigermenge.begin(), zeigermenge.end(),
		bind2nd(WPBez(), "SAP"));
	if (iter != zeigermenge.end())
		cout << "Gefunden: " << **iter << endl;
}
