#include <complex>
#include <iostream>
#include <iomanip>
using namespace std;

template<class T>
class Z {
public:
	explicit Z(const complex<T>& cc) : c(cc) { }
	friend ostream& operator<<(ostream& os, const Z& z) {
		const double eps = 1E-10;
		os << z.c.real();
		if (fabs(z.c.imag()) > eps) {            // z.c.imag() != 0
			os << ((z.c.imag() > 0) ? '+' : '-');
			const T i = fabs(z.c.imag());
			if (fabs(i - 1.0) > eps)               // z.c.imag() != 1
				os << i;
			os << 'i';
		}
		return os;
	}
private:
	const complex<T> c;
};

template<class T>
inline Z<T> z(const complex<T>& c) { return Z<T>(c); }

int main() {
	cout << z(complex<double>(1.2, 0)) << endl;
	cout << z(complex<double>(-1.2, -1)) << endl;
	cout << z(complex<float>(-1.2F, 0.5F)) << endl;
	cout << z(complex<float>(1.2F, -0.5F)) << endl;
}
